# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/bld/qgis_1572244636921/work/python/plugins/db_manager/ui/DlgImportVector.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DbManagerDlgImportVector(object):
    def setupUi(self, DbManagerDlgImportVector):
        DbManagerDlgImportVector.setObjectName("DbManagerDlgImportVector")
        DbManagerDlgImportVector.resize(482, 627)
        self.verticalLayout = QtWidgets.QVBoxLayout(DbManagerDlgImportVector)
        self.verticalLayout.setObjectName("verticalLayout")
        self.wdgInput = QtWidgets.QWidget(DbManagerDlgImportVector)
        self.wdgInput.setObjectName("wdgInput")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.wdgInput)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_5 = QtWidgets.QLabel(self.wdgInput)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_5.sizePolicy().hasHeightForWidth())
        self.label_5.setSizePolicy(sizePolicy)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 0, 0, 1, 1)
        self.btnChooseInputFile = QtWidgets.QToolButton(self.wdgInput)
        self.btnChooseInputFile.setObjectName("btnChooseInputFile")
        self.gridLayout_3.addWidget(self.btnChooseInputFile, 0, 3, 1, 1)
        self.line = QtWidgets.QFrame(self.wdgInput)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_3.addWidget(self.line, 2, 0, 1, 4)
        self.cboInputLayer = QtWidgets.QComboBox(self.wdgInput)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cboInputLayer.sizePolicy().hasHeightForWidth())
        self.cboInputLayer.setSizePolicy(sizePolicy)
        self.cboInputLayer.setEditable(True)
        self.cboInputLayer.setInsertPolicy(QtWidgets.QComboBox.NoInsert)
        self.cboInputLayer.setObjectName("cboInputLayer")
        self.gridLayout_3.addWidget(self.cboInputLayer, 0, 1, 1, 2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.chkSelectedFeatures = QtWidgets.QCheckBox(self.wdgInput)
        self.chkSelectedFeatures.setObjectName("chkSelectedFeatures")
        self.horizontalLayout_2.addWidget(self.chkSelectedFeatures)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.btnUpdateInputLayer = QtWidgets.QPushButton(self.wdgInput)
        self.btnUpdateInputLayer.setObjectName("btnUpdateInputLayer")
        self.horizontalLayout_2.addWidget(self.btnUpdateInputLayer)
        self.gridLayout_3.addLayout(self.horizontalLayout_2, 1, 0, 1, 4)
        self.verticalLayout.addWidget(self.wdgInput)
        self.groupBox_3 = QtWidgets.QGroupBox(DbManagerDlgImportVector)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox_3)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 0, 0, 1, 1)
        self.cboSchema = QtWidgets.QComboBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cboSchema.sizePolicy().hasHeightForWidth())
        self.cboSchema.setSizePolicy(sizePolicy)
        self.cboSchema.setObjectName("cboSchema")
        self.gridLayout_2.addWidget(self.cboSchema, 0, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.groupBox_3)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 1, 0, 1, 1)
        self.cboTable = QtWidgets.QComboBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cboTable.sizePolicy().hasHeightForWidth())
        self.cboTable.setSizePolicy(sizePolicy)
        self.cboTable.setEditable(True)
        self.cboTable.setInsertPolicy(QtWidgets.QComboBox.NoInsert)
        self.cboTable.setObjectName("cboTable")
        self.gridLayout_2.addWidget(self.cboTable, 1, 1, 1, 1)
        self.cboTable.raise_()
        self.label_2.raise_()
        self.cboSchema.raise_()
        self.label_3.raise_()
        self.verticalLayout.addWidget(self.groupBox_3)
        self.groupBox_2 = QtWidgets.QGroupBox(DbManagerDlgImportVector)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.chkDropTable = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkDropTable.setObjectName("chkDropTable")
        self.gridLayout.addWidget(self.chkDropTable, 5, 0, 1, 2)
        self.chkPrimaryKey = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkPrimaryKey.setObjectName("chkPrimaryKey")
        self.gridLayout.addWidget(self.chkPrimaryKey, 0, 0, 1, 1)
        self.editPrimaryKey = QtWidgets.QLineEdit(self.groupBox_2)
        self.editPrimaryKey.setEnabled(False)
        self.editPrimaryKey.setObjectName("editPrimaryKey")
        self.gridLayout.addWidget(self.editPrimaryKey, 0, 1, 1, 1)
        self.chkGeomColumn = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkGeomColumn.setObjectName("chkGeomColumn")
        self.gridLayout.addWidget(self.chkGeomColumn, 1, 0, 1, 1)
        self.editGeomColumn = QtWidgets.QLineEdit(self.groupBox_2)
        self.editGeomColumn.setEnabled(False)
        self.editGeomColumn.setObjectName("editGeomColumn")
        self.gridLayout.addWidget(self.editGeomColumn, 1, 1, 1, 1)
        self.chkEncoding = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkEncoding.setObjectName("chkEncoding")
        self.gridLayout.addWidget(self.chkEncoding, 4, 0, 1, 1)
        self.cboEncoding = QtWidgets.QComboBox(self.groupBox_2)
        self.cboEncoding.setEnabled(False)
        self.cboEncoding.setEditable(True)
        self.cboEncoding.setObjectName("cboEncoding")
        self.gridLayout.addWidget(self.cboEncoding, 4, 1, 1, 1)
        self.chkSinglePart = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkSinglePart.setObjectName("chkSinglePart")
        self.gridLayout.addWidget(self.chkSinglePart, 7, 0, 1, 2)
        self.chkSpatialIndex = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkSpatialIndex.setObjectName("chkSpatialIndex")
        self.gridLayout.addWidget(self.chkSpatialIndex, 9, 0, 1, 2)
        self.chkLowercaseFieldNames = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkLowercaseFieldNames.setObjectName("chkLowercaseFieldNames")
        self.gridLayout.addWidget(self.chkLowercaseFieldNames, 8, 0, 1, 2)
        self.chkTargetSrid = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkTargetSrid.setObjectName("chkTargetSrid")
        self.gridLayout.addWidget(self.chkTargetSrid, 3, 0, 1, 1)
        self.widgetTargetSrid = QgsProjectionSelectionWidget(self.groupBox_2)
        self.widgetTargetSrid.setEnabled(False)
        self.widgetTargetSrid.setObjectName("widgetTargetSrid")
        self.gridLayout.addWidget(self.widgetTargetSrid, 3, 1, 1, 1)
        self.widgetSourceSrid = QgsProjectionSelectionWidget(self.groupBox_2)
        self.widgetSourceSrid.setEnabled(False)
        self.widgetSourceSrid.setObjectName("widgetSourceSrid")
        self.gridLayout.addWidget(self.widgetSourceSrid, 2, 1, 1, 1)
        self.chkSourceSrid = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkSourceSrid.setObjectName("chkSourceSrid")
        self.gridLayout.addWidget(self.chkSourceSrid, 2, 0, 1, 1)
        self.chkCom = QtWidgets.QCheckBox(self.groupBox_2)
        self.chkCom.setObjectName("chkCom")
        self.gridLayout.addWidget(self.chkCom, 10, 0, 1, 1)
        self.editCom = QtWidgets.QLineEdit(self.groupBox_2)
        self.editCom.setEnabled(False)
        self.editCom.setObjectName("editCom")
        self.gridLayout.addWidget(self.editCom, 10, 1, 1, 1)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.buttonBox = QtWidgets.QDialogButtonBox(DbManagerDlgImportVector)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.groupBox_2.raise_()
        self.buttonBox.raise_()
        self.groupBox_3.raise_()
        self.wdgInput.raise_()

        self.retranslateUi(DbManagerDlgImportVector)
        self.buttonBox.rejected.connect(DbManagerDlgImportVector.reject)
        self.chkPrimaryKey.toggled['bool'].connect(self.editPrimaryKey.setEnabled)
        self.chkGeomColumn.toggled['bool'].connect(self.editGeomColumn.setEnabled)
        self.chkEncoding.toggled['bool'].connect(self.cboEncoding.setEnabled)
        self.buttonBox.accepted.connect(DbManagerDlgImportVector.accept)
        self.chkSourceSrid.toggled['bool'].connect(self.widgetSourceSrid.setEnabled)
        self.chkTargetSrid.toggled['bool'].connect(self.widgetTargetSrid.setEnabled)
        self.chkCom.toggled['bool'].connect(self.editCom.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(DbManagerDlgImportVector)
        DbManagerDlgImportVector.setTabOrder(self.cboInputLayer, self.btnChooseInputFile)
        DbManagerDlgImportVector.setTabOrder(self.btnChooseInputFile, self.chkSelectedFeatures)
        DbManagerDlgImportVector.setTabOrder(self.chkSelectedFeatures, self.btnUpdateInputLayer)
        DbManagerDlgImportVector.setTabOrder(self.btnUpdateInputLayer, self.cboSchema)
        DbManagerDlgImportVector.setTabOrder(self.cboSchema, self.cboTable)
        DbManagerDlgImportVector.setTabOrder(self.cboTable, self.chkPrimaryKey)
        DbManagerDlgImportVector.setTabOrder(self.chkPrimaryKey, self.editPrimaryKey)
        DbManagerDlgImportVector.setTabOrder(self.editPrimaryKey, self.chkGeomColumn)
        DbManagerDlgImportVector.setTabOrder(self.chkGeomColumn, self.editGeomColumn)
        DbManagerDlgImportVector.setTabOrder(self.editGeomColumn, self.chkSourceSrid)
        DbManagerDlgImportVector.setTabOrder(self.chkSourceSrid, self.chkTargetSrid)
        DbManagerDlgImportVector.setTabOrder(self.chkTargetSrid, self.chkEncoding)
        DbManagerDlgImportVector.setTabOrder(self.chkEncoding, self.cboEncoding)
        DbManagerDlgImportVector.setTabOrder(self.cboEncoding, self.chkDropTable)
        DbManagerDlgImportVector.setTabOrder(self.chkDropTable, self.chkSinglePart)
        DbManagerDlgImportVector.setTabOrder(self.chkSinglePart, self.chkLowercaseFieldNames)
        DbManagerDlgImportVector.setTabOrder(self.chkLowercaseFieldNames, self.chkSpatialIndex)

    def retranslateUi(self, DbManagerDlgImportVector):
        _translate = QtCore.QCoreApplication.translate
        DbManagerDlgImportVector.setWindowTitle(_translate("DbManagerDlgImportVector", "Import vector layer"))
        self.label_5.setText(_translate("DbManagerDlgImportVector", "Input"))
        self.btnChooseInputFile.setText(_translate("DbManagerDlgImportVector", "…"))
        self.chkSelectedFeatures.setText(_translate("DbManagerDlgImportVector", "Import only selected features"))
        self.btnUpdateInputLayer.setText(_translate("DbManagerDlgImportVector", "Update options"))
        self.groupBox_3.setTitle(_translate("DbManagerDlgImportVector", "Output table"))
        self.label_2.setText(_translate("DbManagerDlgImportVector", "Schema"))
        self.label_3.setText(_translate("DbManagerDlgImportVector", "Table"))
        self.groupBox_2.setTitle(_translate("DbManagerDlgImportVector", "Options"))
        self.chkDropTable.setText(_translate("DbManagerDlgImportVector", "Replace destination table (if exists)"))
        self.chkPrimaryKey.setText(_translate("DbManagerDlgImportVector", "Primary key"))
        self.chkGeomColumn.setText(_translate("DbManagerDlgImportVector", "Geometry column"))
        self.chkEncoding.setText(_translate("DbManagerDlgImportVector", "Encoding"))
        self.chkSinglePart.setText(_translate("DbManagerDlgImportVector", "Do not promote to multi-part"))
        self.chkSpatialIndex.setText(_translate("DbManagerDlgImportVector", "Create spatial index"))
        self.chkLowercaseFieldNames.setText(_translate("DbManagerDlgImportVector", "Convert field names to lowercase"))
        self.chkTargetSrid.setText(_translate("DbManagerDlgImportVector", "Target SRID"))
        self.chkSourceSrid.setText(_translate("DbManagerDlgImportVector", "Source SRID"))
        self.chkCom.setText(_translate("DbManagerDlgImportVector", "Comment"))

from qgis.gui import QgsProjectionSelectionWidget
